; -----------------------------------
; Zirkulation.asm
;
; Steuerprogramm fr Zirkulationssteuerung
;
; Schaltet die Zirkulationspumpe einer Heizungsanlage in Abhngigkeit von
; der Zeit und der Temperatur im Rcklauf
; Start & Stopzeit sowie Temperaturschwelle einstellbar
; Mit Echtzeituhr, LED-Display, 3Tasten
; Eingang fr Temperaturfhler KTY81-110
;
; (c) Stephan Asmus 10/2012
; info@sasmus.de
;
; Das Programm oder Teile davon drfen fr private, nicht-kommerzielle
; Anwendungen weiterverwendet werden.
; Der ursprngliche Autor ist dabei zu erwhnen
; -----------------------------------
; PIC 16F690
; -----------------------------------
; Log:
; 13.10.2012 Start
; 15.10.2012 i2c running, time on display, ISR running, beginning menu
; 17.10.2012 Menu running, save/load prefs running
; 18.10.2012 temperature measurement ready, switching within time working
; 19.10.2012 logic implemented, ready, working
; 1st release
;
; Used Controller: PIC 16F690
; Pins & Ports:
;
;   Port C:
;   0: Segment a
;   1: Segment f
;   2: Segment b
;   3: Segment e
;   4: Segment c
;   5: Segment d
;   6: Segment g
;   7: Dots between Display Positions 2 and 3
;
;   Port B:
;   4:  Display 1
;   5:  Display 2
;   6:  Display 3
;   7:  Display 4
;
;   Port A:
;   0:  Temp Sensor analog input
;   1:  Key left/down input
;   2:  Relais output
;   3:  Ley right/up input
;   4:  I2C SCL
;   5:  I2C SDA
;
; see schematic for more Details
;
; Fosc=4MHz internal


	LIST   P=PIC16F690

	#include <p16f690.inc>

    errorlevel  -302		;No bank select warnings
    errorlevel  -306		;No page select warnings

	__config (_INTOSCIO & _WDT_OFF & _PWRTE_OFF & _MCLRE_OFF & _CP_OFF & _BOD_OFF & _IESO_OFF & _FCMEN_OFF)
;----------------------------------------------------------------
;variables
w_tmp		EQU	0x20    ; used for context saving Register "W"
status_tmp	EQU	0x21    ; used for context saving Register "Status"
pclath_tmp	equ	0x22	; used for context saving Register "PCLATH"
pclath_save	equ 0x23	; used to save pclath within computed gotos
	cblock 0x24
	d1				;Delay-Variable 1
	d2				;Delay-Variable 2
	d3				;Delay-Variable 3

    xhi             ;for math routines
    xlo
    yhi
    ylo

    tmp             ;temp variable universal
    tmp1
    temp
    tmpGIE          ;temp GIE
    count           ;used in I2C routines
    f0              ;used by conversion routines

	sec
	min
	hr				;current time in bcd format

    startmin        ;start- & endtimes
    starthr
    endmin
    endhr
    newmin          ;used to set new time (temp register)
    newhr

    treshold        ;holds treshold temperature
    temperature     ;holds current temperature

    disp1           ; holds 4 Display positions (active)
    disp2
    disp3
    disp4
    position        ;Position of Display (0..3)

    dispvalh        ;(bin) Values of Display
    dispvall

    tickcounter_l   ;counter fr ISR loop
    tickcounter_h

    keydata         ;holds pressed key

    flags           ;holds diverse flags (see definitions)

	eprom_adr		;eeprom Adress tmp
	eprom_dat		;eeprom data tmp

	endc
;----------------------------------------------------------------
;constants
#define Relais  PORTA,2	;LED
#define SDA		PORTA,5	;SDA
#define SCL		PORTA,4	;SCL
#define SDA_T	TRISA,5	;Tris-Bit for SDA
#define SCL_T	TRISA,4	;Tris-Bit for SCL
#define newtime flags,0 ;indicates that new time value was loaded
#define intime  flags,1 ;indicates that we are between start & endtime
#define lowtemp flags,2 ;indicates that temperature is below treshold
#define config_changed  flags,3 ;indicates if config has to be saved to eeprom
#define time_changed    flags,4 ;indicates that time was set in Menu "Time"

;----------------------------------------------------------------
;macro definitions
;----------------------------------------------------------------
;set to Register Bank 0
SET_BANK0 MACRO
	bcf STATUS, RP0
	bcf STATUS, RP1
	ENDM
;set to Register Bank 1
SET_BANK1 MACRO
	bsf	STATUS, RP0
	bcf	STATUS, RP1
	ENDM
;set to Register Bank 2
SET_BANK2 MACRO
	bcf	STATUS, RP0
	bsf	STATUS, RP1
	ENDM
;strout Macro to output string on display
strout	macro	stringaddr
    movlw   high stringaddr
    movwf   PCLATH
    call    stringaddr
    movwf   disp1
    call    stringaddr+1
    movwf   disp2
    call    stringaddr+2
    movwf   disp3
    call    stringaddr+3
    movwf   disp4
    movlw   high $
    movwf   PCLATH
		endm
; **********************************************************************
;Program Start
; **********************************************************************
	org 0
	goto    INIT
	ORG     0x004           ; interrupt vector location
	movwf   w_tmp          ; save off current W register contents
	swapf   STATUS,W        ; move status register into W register
	clrf    STATUS          ; select Bank 0
	movwf   status_tmp     ; save off contents of STATUS register
	movf	PCLATH,w
	movwf	pclath_tmp
	clrf	PCLATH

	btfsc   INTCON,T0IF     ; If Timer 0 caused the interrupt, handle it.
	call    Timer0_ISR		; 

	movf	pclath_tmp,w
	movwf	PCLATH
	swapf   status_tmp,w     ; retrieve copy of STATUS register
	movwf   STATUS            ; restore pre-isr STATUS register contents
	swapf   w_tmp,f
	swapf   w_tmp,w          ; restore pre-isr W register contents
	retfie                    ; return from interrupt

; *********************************************
; Timer 0 Interrupt handler.
; Timer 0 has overflowed
; *********************************************
Timer0_ISR

    incf    position,f
    movlw   0x04
    xorwf   position,w
    btfsc   STATUS,Z
    clrf    position

    movf    position,w
    xorlw   0x00
    skpnz
    goto    is_1
    xorlw   0x01^0x00
    skpnz
    goto    is_2
    xorlw   0x02^0x01
    skpnz
    goto    is_3
    goto    is_4

is_1
    clrf    PORTB
    movf    disp1,w
    movwf   PORTC
    bsf     PORTB,4
    goto    disp_loaded
is_2
    clrf    PORTB
    movf    disp2,w
    movwf   PORTC
    bsf     PORTB,5
    goto    disp_loaded
is_3
    clrf    PORTB
    movf    disp3,w
    movwf   PORTC
    bsf     PORTB,6
    goto    disp_loaded
is_4
    clrf    PORTB
    movf    disp4,w
    movwf   PORTC
    bsf     PORTB,7

disp_loaded
    
    incf    tickcounter_l,f     ;increment low-counter (256 ticks ca. 1sec)
    skpz                        ;overflow?
    goto    tmr0end             ;no, then end
    incf    tickcounter_h,f     ;yes, then increment high-couner (1Tick ca. 1sec)
    movf    tickcounter_h,w
    xorlw   0x0A                ;10 secs over?
    skpz
    goto    tmr0end             ;no, then end
    ;else...
    clrf    tickcounter_h       ;clear counter
    call	rtc_brst_rd         ;get time
    bsf     newtime             ;indicate that time has to be updated

    ;measure temperature....
    ;temperature= (measured_value -237)*5 / 8
    bsf     ADCON0,1            ;start conversion
    btfsc   ADCON0,1            ;wait until ready
    goto    $-1
    SET_BANK1
    movf    ADRESL,w
    SET_BANK0
    movwf   temperature
    movlw   d'237'               ;-237
    subwf   temperature,f
    movf    temperature,w
    movwf   tmp1
    rrf     tmp1,f              ;temperature/2
    rrf     tmp1,w              ;temperature/4
    andlw   0x3f                ;mask bit 7 & 6
    addwf   temperature,f
    bcf     STATUS,C            ;clear carry
    rrf     temperature,f       ;/2
    ;now temperature holds (measured value-237)*1,25/2 (*5/8)
    movlw   d'99'               ;check if temp >99C (possibly an error)
    subwf   temperature,w
    btfss   STATUS,C
    goto    tmr0_1              ;no then all fine
    movlw   d'99'               ;yes, then set to max (99C)
    movwf   temperature
tmr0_1
    ;Check intime
    bcf     intime              ;first reset status...
    movf    starthr,w
    movwf   xhi
    movf    startmin,w
    movwf   xlo
    movf    hr,w
    movwf   yhi
    movf    min,w
    movwf   ylo
    call    compare_unsigned_16
    btfss   STATUS,C            ;if carry, then time >=starttime
    goto    tmr0_2              ;then we are not in time

    movf    endhr,w
    movwf   xhi
    movf    endmin,w
    movwf   xlo
    call    compare_unsigned_16
    btfss   STATUS,C            ;if no carry, then time < endtime
    bsf     intime              ;then we're in time!
tmr0_2

    btfsc   intime              ;are w in time?
    goto    tmr0_3              ;yes, then checkl temperature
    bcf     Relais              ;otherwise shut Relais off
    goto    tmr0_5              ;and end isr

tmr0_3
    btfsc   Relais              ;Relais on?
    goto    tmr0_4              ;yes, then other check
    ;we get here if Relais is off
    movlw   0x05                ;low treshold is 5 below treshold
    subwf   treshold,w
    subwf   temperature,w       ;carry is 0 if temperature below treshold-5
    btfsc   STATUS,C
    goto    tmr0_5             ;exit, leave Relais off
    bsf     Relais              ;exit but switch Relais on
    goto    tmr0_5

tmr0_4
    ;we get here if Relais is on
    movf    treshold,w
    subwf   temperature,w
    btfsc   STATUS,C            ;carry is 1 if temperature >=treshold
    bcf     Relais              ;if c=1 then set Relais off
    ;otherwiese leave it on...
tmr0_5

tmr0end
	BCF 	INTCON,T0IF		; Clear Timer0 flag and continue.
    return
;------------------------------------------------------------------
;initializes Processor
;------------------------------------------------------------------
INIT
	SET_BANK1	 		;select Register Page 1
	bsf		OSCCON,IRCF2	;select 4MHz
	bsf		OSCCON,IRCF1
	bcf		OSCCON,IRCF0
    clrf    TRISC           ;Port C all output
    clrf    TRISB           ;Port B all output
    bsf     TRISA,4         ;SCL input
    bsf     TRISA,5         ;SDA input
    bsf     TRISA,0         ;temp Sensor input
    bcf     TRISA,2         ;Relais output
    bsf     TRISA,1         ;key 1 input
    bsf     TRISA,3         ;key 2 input
    bsf		WPUA,WPUA1	;weak pull up keys
    SET_BANK2
    ;configure analog I/O
	clrf 	ANSEL		;all digital I/O
	clrf	ANSELH
    bsf     ANSEL,ANS0  ;besides AN0 for temp sensor
    SET_BANK1
	movlw	b'00010000'		;Clock=Fosc/8
	movwf	ADCON1			;
	SET_BANK0
	movlw	b'10000001'		;configure AD: right just; Vref=VDD
	movwf	ADCON0			;Channel 0; AD=on


    ;initialise Timer 0
	;we are in Bank 0...
	clrwdt				;Routine from Datasheet Microchip
	clrf	TMR0
	bsf		STATUS, RP0	;Switch to BANK 1
	movlw	b'01010011'		;Pullup enable, Prescaler to TMR0, Prescaler 1:16
	movwf   OPTION_REG
	SET_BANK0

	;----now set reasonable values to variables
    clrf    position
    clrf    disp1
    clrf    disp2
    clrf    disp3
    clrf    disp4
    bcf     Relais
	bsf		SDA
	bsf		SCL			;SCL and SDA high
    clrf    tickcounter_l
    clrf    tickcounter_h
    clrf    flags
    clrf    temperature
    
    call    load_prefs

    call    Delay100ms  ;wait for I2C to stabilize...

	;--now get current time
	call	i2c_stop
	call	rtc_brst_rd		;get time
	call	rtc_brst_wr		;save to make sure 24 mode & OSC is running
	call	rtc_init		;to set control reg

    bsf     newtime         ;indicate that time has to be updated

	;---
	clrf 	TMR0
	clrf	INTCON			;initialize Interrupts
	bsf		INTCON,T0IE		;enable Timer0 interrupts
	bsf 	INTCON,GIE		;enable global interrupts
							;now timer 0 fires
;#############################################################################
;here to start main routine###################################################
;#############################################################################
main
    btfss   newtime         ;new time loaded?
    goto    main1
    movf    min,w
    movwf   dispvall
    movf    hr,w
    movwf   dispvalh
    call    value2display   ;yes, then write to display
    bcf     newtime         ;and clear flag

main1
    call    keycheck
    xorlw   0x03            ;ok pressed?
    skpnz
    goto    main2           ;no, then do handle menu
    xorlw   0x01^0x03       ;down pressed?
    skpnz
    goto    main3           ;then handle temp-Display
    goto    main

main2
    call    handle_menu     ;else goto menu and handle it
    call    waitforrelease
    btfsc   config_changed
    call    save_prefs
    goto    main

main3
    movlw   0x05
    movwf   tickcounter_h    ;no update the next 5 secs
    strout	msg_Celsius     ;display " C "
    movf    temperature,w      ;show current temperature
    call    bin2bcd
    movwf   tmp1
    movlw   high numbers
    movwf   PCLATH
    movf    tmp1,w
    andlw   0x0f
    call    numbers
    movwf   disp2
    swapf   tmp1,w      ;show current temperature
    andlw   0x0f
    call    numbers
    movwf   disp1
    movlw   high $
    movwf   PCLATH
    call    waitforrelease
    goto    main
;------------------------------------------------------------------------
;handle_menu
;------------------------------------------------------------------------
handle_menu
    bsf     newtime         ;preset, if menu is left before isr sets flag
    strout	msg_Ready
    call    waitforrelease
	call	getkey			;get new status
	XORLW	0x03			;ok? (exit)
	skpnz
	return					;then leave mainmenu
	xorlw	0x02^0x03		;up pressed?
	skpnz
	goto	hm_start
    goto    handle_menu
hm_start
    strout	msg_Start
    call    waitforrelease
	call	getkey			;get new status
	XORLW	0x01			;down pressed?
	skpnz
	goto    handle_menu 	;then mainmenu
	xorlw	0x02^0x01		;up pressed?
	skpnz
	goto	hm_end
	;we get here if ok was pressed
    goto    handle_starttime
hm_end
    strout	msg_Ende
    call    waitforrelease
	call	getkey			;get new status
	XORLW	0x01			;down pressed?
	skpnz
	goto    hm_start     	;then one back
	xorlw	0x02^0x01		;up pressed?
	skpnz
	goto	hm_clock
	;we get here if ok was pressed
    goto    handle_endtime
hm_clock
    strout	msg_Time
    call    waitforrelease
	call	getkey			;get new status
	XORLW	0x01			;down pressed?
	skpnz
	goto    hm_end       	;then one back
	xorlw	0x02^0x01		;up pressed?
	skpnz
	goto	hm_temp
	;we get here if ok was pressed
    goto    handle_clock
hm_temp
    strout	msg_Temp
    call    waitforrelease
	call	getkey			;get new status
	XORLW	0x01			;down pressed?
	skpnz
	goto    hm_clock       	;then one back
	xorlw	0x03^0x01		;ok pressed?
	skpnz
	goto	handle_temp
	;we get here if up was pressed
    goto    hm_temp
;------------------ 2nd Level
handle_starttime
    movf    startmin,w      ;show current starttime
    movwf   dispvall
    movf    starthr,w
    movwf   dispvalh
    call    value2display

    call    waitforrelease
	call	getkey			;get new status
	XORLW	0x01			;down pressed?
	skpnz
	goto    handle_starttime_hr
	xorlw	0x02^0x01		;up pressed?
	skpnz
    goto    handle_starttime_min
    ;we get here if ok was pressed
    goto    hm_start
handle_starttime_hr
    movf    starthr,w       ;hr to w
    movwf   f0
    call    bcd2bin         ;convert to binary
    addlw   0x01            ;increment
    movwf   tmp1            ;save
    xorlw   d'24'            ;is it 24?
    skpnz
    clrf    tmp1            ;yes, then set to 0
    movf    tmp1,w          ;and save
    call    bin2bcd
    movwf   starthr
    bsf     config_changed
    goto    handle_starttime
handle_starttime_min
    movf    startmin,w       ;hr to w
    movwf   f0
    call    bcd2bin         ;convert to binary
    addlw   0x01            ;increment
    movwf   tmp1            ;save
    xorlw   d'60'            ;is it 60?
    skpnz
    clrf    tmp1            ;yes, then set to 0
    movf    tmp1,w          ;and save
    call    bin2bcd
    movwf   startmin
    bsf     config_changed
    goto    handle_starttime
;------------
handle_endtime
    movf    endmin,w      ;show current endtime
    movwf   dispvall
    movf    endhr,w
    movwf   dispvalh
    call    value2display

    call    waitforrelease
	call	getkey			;get new status
	XORLW	0x01			;down pressed?
	skpnz
	goto    handle_endtime_hr
	xorlw	0x02^0x01		;up pressed?
	skpnz
    goto    handle_endtime_min
    ;we get here if ok was pressed
    goto    hm_end
handle_endtime_hr
    movf    endhr,w       ;hr to w
    movwf   f0
    call    bcd2bin         ;convert to binary
    addlw   0x01            ;increment
    movwf   tmp1            ;save
    xorlw   d'24'            ;is it 24?
    skpnz
    clrf    tmp1            ;yes, then set to 0
    movf    tmp1,w          ;and save
    call    bin2bcd
    movwf   endhr
    bsf     config_changed
    goto    handle_endtime
handle_endtime_min
    movf    endmin,w       ;min to w
    movwf   f0
    call    bcd2bin         ;convert to binary
    addlw   0x01            ;increment
    movwf   tmp1            ;save
    xorlw   d'60'            ;is it 60?
    skpnz
    clrf    tmp1            ;yes, then set to 0
    movf    tmp1,w          ;and save
    call    bin2bcd
    movwf   endmin
    bsf     config_changed
    goto    handle_endtime
;------------
handle_clock
    movf    hr,w            ;copy current time to temp register
    movwf   newhr
    movf    min,w
    movwf   newmin
handle_clock_1
    movf    newmin,w      ;show current starttime
    movwf   dispvall
    movf    newhr,w
    movwf   dispvalh
    call    value2display

    call    waitforrelease
	call	getkey			;get new status
	XORLW	0x01			;down pressed?
	skpnz
	goto    handle_clock_hr
	xorlw	0x02^0x01		;up pressed?
	skpnz
    goto    handle_clock_min
    ;we get here if ok was pressed
    clrf    tickcounter_h   ;no isr driven time-Update next 10 secs...
    btfss   time_changed    ;was time modified?
    goto    hm_clock         ;no, then leave to mainmenu
    bcf     time_changed    ;else clear flag
    movf    newmin,w
    movwf   min
    movf    newhr,w
    movwf   hr
    clrf	sec				;and we want to start a new minute when set
	call	rtc_brst_wr		;write to rtc
    goto    hm_clock         ;leave to Mainmenu
handle_clock_hr
    movf    newhr,w       ;hr to w
    movwf   f0
    call    bcd2bin         ;convert to binary
    addlw   0x01            ;increment
    movwf   tmp1            ;save
    xorlw   d'24'            ;is it 24?
    skpnz
    clrf    tmp1            ;yes, then set to 0
    movf    tmp1,w          ;and save
    call    bin2bcd
    movwf   newhr
    bsf     time_changed
    goto    handle_clock_1
handle_clock_min
    movf    newmin,w       ;min to w
    movwf   f0
    call    bcd2bin         ;convert to binary
    addlw   0x01            ;increment
    movwf   tmp1            ;save
    xorlw   d'60'            ;is it 60?
    skpnz
    clrf    tmp1            ;yes, then set to 0
    movf    tmp1,w          ;and save
    call    bin2bcd
    movwf   newmin
    bsf     time_changed
    goto    handle_clock_1
handle_temp
    strout	msg_Celsius         ;display " C "
    movf    treshold,w      ;show current temperature treshold
    call    bin2bcd
    movwf   tmp1
    movlw   high numbers
    movwf   PCLATH
    movf    tmp1,w
    andlw   0x0f
    call    numbers
    movwf   disp2
    swapf   tmp1,w      ;show current temperature
    andlw   0x0f
    call    numbers
    movwf   disp1
    movlw   high $
    movwf   PCLATH

    call    waitforrelease
	call	getkey			;get new status
	XORLW	0x01			;down pressed (modifier for value)?
	skpnz
	goto    handle_temp_value   ;yes, then handle temperature change
	xorlw	0x03^0x01		;ok pressed?
	skpnz
    goto    hm_temp         ;yes, then back to mainmenu
    ;we get here if up was pressed
    goto    handle_temp
handle_temp_value
    movf    treshold,w       ;temperature to w
    addlw   0x01                ;increment
    movwf   tmp1                ;save
    xorlw   d'71'               ;is it >70?
    skpz
    goto    handle_temp_value_1 ;no
    movlw   d'20'               ;yes, then set to 20
    movwf   tmp1
handle_temp_value_1
    movf    tmp1,w              ;and save
    movwf   treshold
    bsf     config_changed      ;indicate that configuration has changed
    goto    handle_temp         ;and back
;--------------------------------------------------------------------
;value2display
;sets disp1...4 to current values (e.g. time)
;--------------------------------------------------------------------
value2display
    movlw   high numbers
    movwf   PCLATH

    movf	dispvall,w			;load minutes
    andlw   0x0f            ;ones...
    call    numbers
    movwf   disp4
    swapf   dispvall,w           ;tens....
    andlw   0x0f
    call    numbers
    movwf   disp3

    movf	dispvalh,w			;load minutes
    andlw   0x0f
    call    numbers
    movwf   disp2
    swapf   dispvalh,w
    andlw   0x0f
    call    numbers
    movwf   disp1

    bsf     disp1,7             ;this routine is only uded for time-values,
    bsf     disp2,7             ;so switch Points on
    bsf     disp3,7
    bsf     disp4,7

    movlw   high $
    movwf   PCLATH

    return
;--------------------------------------------------------------------
;Math routines
;--------------------------------------------------------------------
;compare unsigned 16
;x=y  --> Z=1
;y<x  --> C=0
;x<=y --> C=1
;--------------------------------------------------------------------
compare_unsigned_16
    movf    xhi,w
    subwf   yhi,w           ;y-x
    skpz                    ;are they equal?
    return                  ;no, then ready
    ;otherwise compare low
    movf    xlo,w
    subwf   ylo,w           ;y-x
    return
;--------------------------------------------------------------------
;Conversion routines
;--------------------------------------------------------------------
;--------------------------------------------------------------------
; BCD to Binary
; input:f0
; output:w
; From: Scott Dattalo
;--------------------------------------------------------------------
bcd2bin
        swapf   f0, W
        andlw   0x0F            ; W=tens
        movwf   tmp
        addwf   tmp, W         ; W=2*tens
        addwf   tmp, F         ; tmp=3*tens (note carry is cleared)
        rlf     tmp, W         ; W=6*tens
        subwf   f0, W          ; W = 16*tens+ones - 6*tens

        return
;--------------------------------------------------------------------
; bin to BCD conversion - limited to 0-99 decimal (0x00-0x63 hex)
; Call with: number to be converted in W
; Returns with: converted BCD number in W
; Registers: tmp, tmp1 used during conversion
;--------------------------------------------------------------------
bin2bcd
		movwf	tmp		;original hex, save to tmp
		clrw
		btfsc	tmp, 4		;if 16 then bcd+16
		addlw	0x16
		btfsc	tmp, 5		;if 32 then bcd+32
		addlw	0x32
		addlw	0x06		;add 6 to correct bcd
		btfss	STATUS,DC	;overflow? then skip
		addlw	-0x06		;else subtract 6 again
		btfsc	tmp, 6		;if 64 then bcd+64
		addlw	0x64
		addlw	0x06		;bcd correction
		btfss	STATUS,DC
		addlw	-0x06
		movwf	tmp1		;save w to tmp1
		movf	tmp, 0		;load original hex to w
		andlw	0x0F		;mask ones
		addwf	tmp1, 0	;add ones to tmp1
		btfsc	STATUS,DC
		addlw	0x06		;bcd correction
		addlw	0x06		;bcd correction again
		btfss	STATUS,DC
		addlw	-0x06
		return
;--------------------------------------------------------------------
;RTC Subroutines
;DS1307 (Real time clock chip with I2C interface)
;--------------------------------------------------------------------
rtc_init
	bcf		tmpGIE,0
	btfss	INTCON,GIE
	goto	rtc_init_1
	bsf		tmpGIE,0		;remember State of GIE
	bcf		INTCON,GIE		;we don't want interrupts now
	btfsc	INTCON,GIE		;make sure it is really cleared
	goto	$-2
rtc_init_1
	call	i2c_start
	movlw	0xD0			; slave address + write
	call	i2c_write
	movlw	0x07			; set word address to control register
	call	i2c_write
	movlw	0x00				;write 0 to control reg
	call	i2c_write
	call	i2c_stop
	btfsc	tmpGIE,0			;restore GIE
	bsf		INTCON,GIE
	return

rtc_brst_rd
	bcf     tmpGIE,0
	btfss	INTCON,GIE
	goto	rtc_brst_rd_1
	bsf     tmpGIE,0		;remember State of GIE
	bcf		INTCON,GIE		;we don't want interrupts now
	btfsc	INTCON,GIE		;make sure it is really cleared
	goto	$-2
rtc_brst_rd_1

	call	i2c_start
	movlw	0xD0			; slave address + write
	call	i2c_write
	movlw	0x00			; set word address to seconds register
	call	i2c_write
	call	i2c_start
	movlw	0xD1			; slave address + read
	call	i2c_write
	call	i2c_read		; read the seconds data
	andlw	0x7f		;set bit 7 to 0
	movwf	sec			; save it
	call	i2c_ack
	call	i2c_read		; and so on
	movwf	min
	call	i2c_ack
	call	i2c_read
	andlw	0x3f		;set bit 7 and 6 to 0
	movwf	hr
	call	i2c_nack
	call	i2c_stop
	btfsc	tmpGIE,0			;restore GIE
	bsf     INTCON,GIE
	return

rtc_brst_wr
	bcf		tmpGIE,0
	btfss	INTCON,GIE
	goto	rtc_brst_wr_1
	bsf		tmpGIE,0		;remember State of GIE
	bcf		INTCON,GIE		;we don't want interrupts now
	btfsc	INTCON,GIE		;make sure it is really cleared
	goto	$-2
rtc_brst_wr_1

	call	i2c_start
	movlw	0xD0			; slave address + write
	call	i2c_write
	movlw	0x00			; set word address to seconds register
	call	i2c_write
	movf	sec, W
	call	i2c_write
	movf	min, W
	call	i2c_write
	movf	hr, W
	call	i2c_write
	call	i2c_stop
	btfsc	tmpGIE,0			;restore GIE
	bsf	INTCON,GIE
	return
;--------------------------------------------------------------------
;I2C Subroutines
;--------------------------------------------------------------------
i2c_start
	bsf		SDA			; SDA high
	;fixme: next two lines necessary?
    ;SET_BANK1
	;bcf		SDA_T		; set SDA for output
	nop
	SET_BANK1
	bsf		SCL_T		; SCL high (input)
	SET_BANK0
	bcf		SDA			; SDA low (start)
	return
;-------------
i2c_stop; assumes SCL is high on entry
	bcf		SDA			; SDA low
	nop
	nop
	SET_BANK1
	bsf		SCL_T		; SCL high (input)
	SET_BANK0
	bsf		SDA			; SDA high
	return
;-------------
;i2c_read
;returns read byte in w
;-------------
i2c_read
	SET_BANK1
	bsf		SDA_T		; set SDA for input
	SET_BANK0

	movlw	08h			; send 8 bits
	movwf	count

	bcf		SCL			; clock data out
	SET_BANK1
	bcf		SCL_T		; SCL low (output)
	SET_BANK0

	clrf	tmp			; clear var
	rlf		tmp,1			; rotate carry in
	clrf	tmp			; clear var again

i2c_read_loop
	rlf		tmp, 1

	SET_BANK1
	bsf		SCL_T		; SCL high (input)
	SET_BANK0

	btfsc	SDA
	bsf		tmp, 0			; if data out = 1, set bit

	bcf		SCL
	SET_BANK1
	bcf		SCL_T		; SCL low (output)
	SET_BANK0
	decfsz	count, 1
	goto	i2c_read_loop

	movf	tmp, W

	return

;---- ACK read (assumes SCL=0 on entry) ----
i2c_ack
	bcf		SDA

	SET_BANK1
	bcf		SDA_T		; set SDA for output
	SET_BANK0

	SET_BANK1
	bsf		SCL_T		; SCL high (input)
	SET_BANK0
	nop
	bcf		SCL
	SET_BANK1
	bcf		SCL_T		; SCL low (output)
	SET_BANK0

	return

;---- NACK read (assumes SCL = 0 on entry) ----
i2c_nack
	bsf		SDA

	SET_BANK1
	bcf		SDA_T		; set SDA for output
	SET_BANK0

	SET_BANK1
	bsf		SCL_T		; SCL high (input)
	SET_BANK0

	bcf	SCL
	SET_BANK1
	bcf		SCL_T		; SCL low (output)
	SET_BANK0

	return

;--- Do a I2C bus write of byte in 'tmp' ---
;
i2c_write
	movwf	tmp		;save tmp

	SET_BANK1
	bcf		SDA_T		; set SDA for output
	SET_BANK0

	movlw	08h			; send 8 bits
	movwf	count

	bcf		SCL
	SET_BANK1
	bcf		SCL_T		; SCL low (output)
	SET_BANK0

i2c_w_loop
	bcf		SDA			; assume data out is low
	btfsc	tmp, 7
	bsf		SDA			; if data out = 1, set bit
	; nop

	SET_BANK1
	bsf		SCL_T		; SCL high (input)
	SET_BANK0
	rlf		tmp, 1
	bcf		SCL			; clock it in
	SET_BANK1
	bcf		SCL_T		; SCL low (output)
	SET_BANK0
	decfsz	count, 1
	goto	i2c_w_loop

	SET_BANK1
	bsf		SDA_T	; set SDA for input
	SET_BANK0

	bcf		SCL
	SET_BANK1
	bcf		SCL_T		; SCL low (output)
	SET_BANK0
	; nop
	SET_BANK1
	bsf		SCL_T		; SCL high (input)
	SET_BANK0
	; if(sda) printf("Ack bit missing  %02X\n",(unsigned int)d);
	bcf		SCL
	SET_BANK1
	bcf		SCL_T		; SCL low (output)
	SET_BANK0

	return
;---------------------------------------------------------------
;load Prefs
;loads Preferences from internal eeprom if valid config is saved
; in eeprom. Set to default values otherwise
;---------------------------------------------------------------
load_prefs
	clrf	eprom_adr		;read pos 0 =valid-check
	call	eprom_read
	xorlw	0x55
	skpz					;go on if config is valid
	goto	load_prefs_def	;set to defaults otherwise
	;config is valid, now load them
	;stati
	call	read_it
	movwf	starthr
	call	read_it
	movwf	startmin
	call	read_it
	movwf	endhr
	call	read_it
	movwf	endmin
	call	read_it
	movwf	treshold
	goto	load_prefs_end

load_prefs_def			;set default to prefs
	;bsf		configerr	;if we got here, config in eeprom is not valid
	;stati
    movlw   0x06
    movwf   starthr
    clrf    startmin
    movlw   0x22
    movwf   endhr
    clrf    endmin
    movlw   d'50'
    movwf   treshold
	goto	load_prefs_end
read_it
	incf	eprom_adr,f		;next
	call	eprom_read
	return
load_prefs_end
	return
;---------------------------------------------------------------
;save_Prefs
; save Preferences to internal eeprom
;---------------------------------------------------------------
save_prefs
	clrf	eprom_adr		;begin saving at pos 0
	movlw	0x55			;valid config marker
	movwf	eprom_dat
	call	eprom_write
	;stati
	movf	starthr,w
	call	save_it
	movf	startmin,w
	call	save_it
	movf	endhr,w
	call	save_it
	movf	endmin,w
	call	save_it
	movf    treshold,w
	call	save_it
	goto	save_prefs_end
save_it
	movwf	eprom_dat
	incf	eprom_adr,f
	call	eprom_write
	return
save_prefs_end
	return
;---------------------------------------------------------------
;EEPROM handling routines
;---------------------------------------------------------------
;eprom read: Address in eprom_adr, return value in w
;---------------------------------------------------------------
eprom_read
	bcf		PIR2, EEIF		;clear interrupt flag (necessary?)
	movf	eprom_adr,w		;Set adress of read
	banksel	EEADR
	movwf	EEADR

	banksel	EECON1
	bcf		EECON1,EEPGD	;we want to read data mem
	bsf		EECON1,RD		;init read
	banksel	EEDATA
	movf	EEDATA,w
	bcf		STATUS,RP0		;back to bank 0 (our standard)
	bcf		STATUS,RP1
	return
;-------------------------------------------------------------------------
;eprom write Address in eprom_adr, data in eprom_dat returns when finished
;-------------------------------------------------------------------------
eprom_write
	bcf		PIR2, EEIF		;clear interrupt (to be sure)
	movf	eprom_adr,w		;load Adress
	banksel	EEADR
	movwf	EEADR
	banksel	0x00
	movf	eprom_dat,w	;load value
	banksel	EEDATA
	movwf	EEDATA
	banksel	EECON1
	bcf		EECON1,EEPGD	;we want to write data mem
	bsf		EECON1,WREN		;enable write

	bcf		INTCON,GIE		;we don't want interrupts now
	btfsc	INTCON,GIE		;make sure it is really cleared
	goto	$-2
	movlw	0x55			;see datasheet, required sequence
	movwf	EECON2
	movlw	0xaa
	movwf	EECON2
	bsf		EECON1,WR		;set write bit to begin write
	bsf		INTCON,GIE		;re-enable global interrupts

	banksel	0x00			;back to Bank0 (our default)
	btfss	PIR2,EEIF		;wait for flag to be set (polling)
	goto	$-1
	bcf		PIR2,EEIF		;clear it, write complete

	banksel	EECON1
	bcf		EECON1,WREN		;clear write enable
	banksel	0x00

	return
;---------------------------------------------------------------
;key handling
;returns status of keys in w
;0= nothing pressed
;3= ok
;2= right
;1= left
;---------------------------------------------------------------
;doesn't wait for anything, just checks current status
keycheck
	movf	PORTA,w
    andlw   b'00001010'     ;mask bit 1 & 3
    xorlw   b'00001010'     ;inverse
    clrf    keydata
    movwf   keydata
    rrf     keydata         ;shift bit1->0 and 3->2
    btfsc   keydata,2
    bsf     keydata,1
    movf    keydata,w
    andlw   b'0000011'
    movwf   keydata
	return
;waits for key to be pressed
getkey
	call 	keycheck
	skpnz						;if no key pressed
	goto 	getkey				;check again
	return						;otherwise return to caller
;waits for release of any key
waitforrelease
	call 	Delay10ms			;debounce
	call	keycheck
	movf	keydata,w
	skpz						;if zero then enough
	goto	waitforrelease
	call 	Delay10ms			;debounce
	return
;--------------------------------------------------------------------
;Delay-Subroutines (for fosc=4 MHz!!!)
;--------------------------------------------------------------------
Delay10ms					;Delays ca.10ms
	movlw	0xCE
	movwf	d1
	movlw	0x08
	movwf	d2
Delay10ms_1
	decfsz	d1, f
	goto	$+2
	decfsz	d2, f
	goto	Delay10ms_1
	goto	$+1
	nop
	return

;-----
Delay100ms					;Delays ca.100ms
	movlw	0x1E
	movwf	d1
	movlw	0x4F
	movwf	d2
Delay100ms_1
	decfsz	d1, f
	goto	$+2
	decfsz	d2, f
	goto	Delay100ms_1
	goto	$+1
	nop
	return
;########################################################################
	org	0x800	;define next in memory Page 1 (we need space in Page 0)
;########################################################################
;---------------------------------------------------------
;Definitions for Display
;Table is short enough to ensure that no rollover from 0xff to 0x00 happens
;within this table - otherwise the code above has to be modified

numbers
    addwf   PCL,f
    dt  0x3F,0x14,0x6D,0x75,0x56,0x73,0x7b,0x15,0x7f,0x77
msg_Ready
    dt  0x48,0x7c,0x76,0x00
msg_Start
    dt  0x73,0x6a,0x48,0x6a
msg_Ende
    dt  0x6b,0x58,0x7c,0x6b
msg_Time
    dt  0x6a,0x14,0x58,0x6b
msg_Temp
    dt  0x6a,0x6b,0x58,0x4f
msg_Celsius
    dt  0x00,0x00,0x47,0x2b


    end
